package com.fr.plugin.third.party.jsdiehc.function;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.cache.Attachment;
import com.fr.general.FArray;
import com.fr.intelli.record.Focus;
import com.fr.intelli.record.Original;
import com.fr.plugin.context.PluginContexts;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.script.AbstractFunction;
import com.fr.stable.Primitive;
import com.fr.stable.fun.Authorize;
import net.coobird.thumbnailator.Thumbnails;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;


/**
 * 图片压缩缩放函数<br/>
 * IMAGE_COMPRESSION_SCALE(image, [quality], [scale])<br/>
 * 参数:<br/>
 * image:图片对象<br/>
 * quality:压缩质量<br/>
 * scale:缩放比例
 */
@Authorize(callSignKey = "com.fr.plugin.third.party.jsdiehc")
@EnableMetrics
public class ImageCompressionScaleFunction extends AbstractFunction {

    @Focus(id = "com.fr.plugin.third.party.jsdiehc", text = "plugin-jsdiehc", source = Original.PLUGIN)
    public Object run(Object[] args) {
        if (!PluginContexts.currentContext().isAvailable()) {
            LogKit.error("图片压缩函数许可证过期");
            return Primitive.ERROR_VALUE;
        }

        if ((args == null) || (args.length <= 0)) {
            return Primitive.ERROR_VALUE;
        }

        if (args.length <= 1) {
            return args;
        }
        Object obj = args[0];
        if (obj == null) {
            return Primitive.ERROR_VALUE;
        }

        Object obj1 = args[1];
        if (obj1 == null) {
            return Primitive.ERROR_VALUE;
        }

        try {
            float quality = Float.valueOf(String.valueOf(obj1));
            if (!(obj instanceof FArray)) {
                return compressImage(obj, quality);
            }

            FArray objArray = (FArray) obj;
            FArray newArray = new FArray();
            if (objArray.length() <= 0) {
                return obj;
            }
            Object tempObj, compressObj;
            for (int i = 0, max = objArray.length() - 1; i <= max; i++) {
                tempObj = objArray.elementAt(i);
                compressObj = compressImage(tempObj, quality);
                newArray.add(compressObj);
            }
            return newArray;
        } catch (Exception e) {
            LogKit.error("jsdiehc图片压缩函数出错," + e.getMessage(), e);
            return Primitive.ERROR_VALUE;
        }
    }

    private Object compressImage(Object obj, float quality) throws IOException {
        if (obj == null) {
            return null;
        }

        if ((quality < 0) || (quality >= 1)) {
            quality = 1;
        }

        if (obj instanceof BufferedImage) {
            BufferedImage inputBufferedImage = (BufferedImage) obj;
            BufferedImage outBufferedImage = Thumbnails.of(inputBufferedImage).scale(1).outputQuality(quality).asBufferedImage();
            return outBufferedImage;
        }

        if (!(obj instanceof Attachment)) {
            return obj;
        }
        Attachment attachment = (Attachment) obj;
        if (!StringKit.equalsIgnoreCase(attachment.getType(), "image")) {
            return obj;
        }

        InputStream inputStream = attachment.getInputStream();
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        Thumbnails.of(inputStream).scale(1).outputQuality(quality).toOutputStream(outputStream);
        ImageAttachment imageAttachment = new ImageAttachment();
        imageAttachment.setImageBytes(outputStream.toByteArray());
        return imageAttachment;
    }

}
